# 部署方式
DEPLOY_MODE_LOCAL = "local"
DEPLOY_MODE_REMOTE = "remote"

# API_NAME
API_CREATE_APP_INSTANCE = "/CreateAppInstance"
API_CREATE_APP_SCENE_META = "/CreateAppSceneMeta"
API_DESCRIBE_APP_INSTANCES = "/DescribeAppInstances"
API_DESCRIBE_SESSION_ANSWER = "/DescribeSessionAnswer"